/*
	$Id: lua.h,v 1.14 2001/09/22 15:52:16 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanLua="Scripting"
//! header=lua.h

#ifndef header_lua_lib
#define header_lua_lib

extern "C" {
#include <lua.h>
#include <lualib.h>
}

/*
The original classes from Waldemer Celes adapted by Lenny Palozzi were replaced by this
to get it to work with Lua and tolua 4.0
jpsousa
*/

int tolua_clanbindings_open (lua_State* tolua_S);
void tolua_clanbindings_close (lua_State* tolua_S);

//: Lua Class
class CL_Lua{
public:
//! Variables:
	//: state
	static lua_State *state;

	//: Initialised
	static bool initialized;

//! Operations:
	//: Init
	static void init()
	{
		state=luaL_newstate();
		tolua_clanbindings_open(state);
		luaopen_base(state);
		initialized=true;
	}

	//: DeInit
	static void deinit()
	{
		tolua_clanbindings_close(state);
		lua_close(state);
		initialized=false;
	}

	//: DoFile
	static int dofile (char *filename)
	{ 
		cl_assert(initialized);
		return luaL_dofile(state,filename);
	}


	//: Open the IO library
	static void iolibopen(void) { luaopen_io(state); }

	//: Open the string library
	static void strlibopen(void)
	{ 
		cl_assert(initialized);
		luaopen_string(state);
	}

	//: Open the math library
	static void mathlibopen(void)
	{ 
		cl_assert(initialized);
		luaopen_math(state);
	}

	//: Open the db library
	static void dblibopen(void)
	{ 
		cl_assert(initialized);
		luaopen_debug(state);
	}
};
#endif








